<?php


class Aop
{
  public $gatewayUrl = 'https://openapi.alipay.com/gateway.do'; // 网关地址
  public $appId; // 应用id
  public $format = 'json'; // 请求参数格式，固定json
  public $charset = 'utf-8'; // 请求使用的编码格式
  public $signType = 'RSA2'; // 签名算法类型，官方推荐RSA2
  public $version = '1.0'; // 调用的接口版本（固定1.0）
  public $timestamp; // 发送请求的时间
  protected $privateKey; // 私钥

  /**
   * 格式化私钥
   * @param $privateKey 私钥字符串
   */
  public function setPrivateKey ($privateKey) {
    $this->privateKey = "-----BEGIN RSA PRIVATE KEY-----\n" .
      chunk_split($privateKey, 64, "\n") .
      "-----END RSA PRIVATE KEY-----";
  }

  /**
   * 获取请求参数数据数组
   * @param $request 订单参数
   * @return array 请求数据
   */
  public function getData ($request) {
    return [
      'app_id' => $this->appId,
      'format' => $this->format,
      'charset' => $this->charset,
      'sign_type' => $this->signType,
      'timestamp' => $this->timestamp,
      'version' => $this->version,
      'method' => $request->method,
      'biz_content' => $request->bizContent,
      'return_url' => $request->returnUrl
    ];
  }

  /**
   * 序列化请求参数数据
   * @param $data 请求参数数据
   * @return string 序列化后的参数
   */
  public function serData ($data) {
    $newData = [];
    foreach ($data as $k => $v) {
//      1、去除参数名和值前后的空格
      $key = trim($k);
      $value = trim($v);
//      2、去除空值参数
      if ($key && $value) {
        $newData[$key] = $value;
      }
    }
//    3、去除sign参数
    unset($newData['sign']);
//    4、按参数名排序
    ksort($newData);
//    5、拼凑序列化参数
    $para = '';
    foreach ($newData as $k => $v) {
      $para .= $k . "=$v&";
    }
    $para = substr($para, 0, -1);
    return $para;
  }

  /**
   * 根据序列化参数数据进行签名
   * @param $para 序列化的参数数据
   * @return bool|string
   */
  public function getSign ($para) {
    openssl_sign($para, $sign, $this->privateKey, OPENSSL_ALGO_SHA256);
    return urlencode(base64_encode($sign));
  }

  /**
   * 解析请求
   * @param $request 订单参数
   */
  public function execute ($request) {
    $data = $this->getData($request);
    $para = $this->serData($data);
    $sign = $this->getSign($para);
//    跳转到支付宝页面
    $url = $this->gatewayUrl . '?' . $para . '&sign=' . $sign;
    header("Location:$url");
  }
}
